@@ -18,7 +18,9 @@ from core import __version__
 # vendor
 import gevent, gevent.socket
 socket = gevent.socket
-
+# protect stats  
+from gevent.thread import allocate_lock as Lock
+stats_lock = Lock()
 
 # constants
 INTERVAL = 10.0
@@ -48,6 +50,7 @@ class Stats(object):
     def __init__(self):
         self.timers = defaultdict(list)
         self.counts = defaultdict(float)
+        self.gauges = defaultdict(float)
         self.percent = PERCENT
         self.interval = INTERVAL
 
@@ -93,7 +96,8 @@ class StatsDaemon(object):
     A statsd service implementation in Python + gevent.
     """
 
-    def __init__(self, bindaddr, sinkspecs, interval, percent, debug=0):
+    def __init__(self, bindaddr, sinkspecs, interval, percent, debug=0,
+                 key_prefix=''):
         _, host, port = parse_addr(bindaddr)
         if port is None:
             self.exit(E_BADADDR % bindaddr)
@@ -123,13 +127,15 @@ class StatsDaemon(object):
         self._debug = debug
         self._sock = None
         self._flush_task = None
+        self._key_prefix = key_prefix
 
         self._reset_stats()
 
     def _reset_stats(self):
-        self._stats = Stats()
-        self._stats.percent = self._percent
-        self._stats.interval = self._interval
+        with stats_lock:
+            self._stats = Stats()
+            self._stats.percent = self._percent
+            self._stats.interval = self._interval
 
     def exit(self, msg, code=1):
         self.error(msg)
@@ -163,7 +169,7 @@ class StatsDaemon(object):
         self._flush_task = gevent.spawn(_flush_impl)
 
         # start accepting connections
-        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, 
+        self._sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM,
             socket.IPPROTO_UDP)
         self._sock.bind(self._bindaddr)
         while 1:
@@ -187,6 +193,8 @@ class StatsDaemon(object):
         # interpret the packet and update stats
         stats = self._stats
         key = parts[0].translate(KEY_TABLE, KEY_DELETIONS)
+        if self._key_prefix:
+            key = '.'.join([self._key_prefix, key])
         for part in parts[1:]:
             srate = 1.0
             fields = part.split('|')
@@ -196,22 +204,26 @@ class StatsDaemon(object):
             value = fields[0]
             stype = fields[1].strip()
 
-            # timer (milliseconds)
-            if stype == 'ms':
-                stats.timers[key].append(float(value if value else 0))
+            with stats_lock:
+                # timer (milliseconds)
+                if stype == 'ms':
+                    stats.timers[key].append(float(value if value else 0))
 
-            # counter with optional sample rate
-            elif stype == 'c':
-                if length == 3 and fields[2].startswith('@'):
-                    srate = float(fields[2][1:])
-                value = float(value if value else 1) * (1 / srate)
-                stats.counts[key] += value
+                # counter with optional sample rate
+                elif stype == 'c':
+                    if length == 3 and fields[2].startswith('@'):
+                        srate = float(fields[2][1:])
+                    value = float(value if value else 1) * (1 / srate)
+                    stats.counts[key] += value
+                elif stype == 'g':
+                    value = float(value if value else 1)
+                    stats.gauges[key] = value
 
 
 def main():
     opts = optparse.OptionParser(description=DESCRIPTION, version=__version__,
         add_help_option=False)
-    opts.add_option('-b', '--bind', dest='bind_addr', default=':8125', 
+    opts.add_option('-b', '--bind', dest='bind_addr', default=':8125',
         help="bind [host]:port (host defaults to '')")
     opts.add_option('-s', '--sink', dest='sink', action='append', default=[],
         help="a graphite service to which stats are sent ([host]:port).")
@@ -219,6 +231,8 @@ def main():
         help="increase verbosity (currently used for debugging)")
     opts.add_option('-f', '--flush', dest='interval', default=INTERVAL,
         help="flush interval, in seconds (default 10)")
+    opts.add_option('-x', '--prefix', dest='key_prefix', default='',
+        help="key prefix added to all keys (default None)")
     opts.add_option('-p', '--percent', dest='percent', default=PERCENT,
         help="percent threshold (default 90)")
     opts.add_option('-D', '--daemonize', dest='daemonize', action='store_true',
@@ -237,10 +251,9 @@ def main():
         daemonize()
 
     sd = StatsDaemon(options.bind_addr, options.sink, options.interval,
-        options.percent, options.verbose)
+                     options.percent, options.verbose, options.key_prefix)
     sd.start()
- 
+
 
 if __name__ == '__main__':
     main()
-
